/***************************************************************************
 *   Copyright (C) 2007 by Francesc Burrull i Mestres.                    *
 *   francesc.burrull@upct.es   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _W2LAN_H_
#define _W2LAN_H_

//#ifdef HAVE_CONFIG_H
//#include <config.h>
//#endif

/* fprintf */
#include <stdio.h>

/* exit */
#include <stdlib.h>

/* getopt */
#include <unistd.h>

/* check if wireless device is in ad-hoc mode */
#include <netdb.h>
#include <linux/wireless.h>

/* libpcap for packet capture */
#include <pcap.h>

/* includes net/ethernet.h */
#include <netinet/if_ether.h>

/* string manipulation */
#include <string.h>

/* isprint */
#include <ctype.h>

/* will see.... */
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <linux/if.h>

/* time */
#include <time.h> 

//     #include <errno.h>
//     #include <sys/types.h>
#include <sys/time.h>

// Alarm for reactive mode
#include <signal.h>

/* program usage */
#ifndef _W2LAN_HELP_H_
#include "w2lan_help.h"
#endif

/* w2lan callback function */
#ifndef _W2LAN_CALLBACK_H_
#include "w2lan_callback.h"
#endif

/* w2lan lists */
#ifndef _W2LAN_LISTS_H_
#include "w2lan_lists.h"
#endif

/* Stable version should be 0 */
#define DEBUG 0

/* Exit codes */
#define SUCCESS 0
#define NO_PARAMETERS -1
#define UNKNOWN_OPTION -1
#define TOO_MANY_ARGUMENTS -1
#define TOO_LITTLE_ARGUMENTS -1
#define IOCTL_ERROR -1
#define ERROR_OPENING_DEVICE -1
#define ERROR_PCAP_DATALINK -1
#define ERROR_DATALINK_TYPE -1
#define NO_WIRELESS_EXTENSIONS -1
#define WIRELESS_DEVICE_NOT_FOUND -1
#define UNKNOWN_WIRELESS_MODE -1
#define ERROR_OPENING_SOCKET -1
#define INVALID_MODE -1
#define INCORRECT_WIRELESS_MODE -1
#define NO_PROC_NET_ROUTE -1
#define UNABLE_GET_MY_MAC -1
#define NEVER_HERE -1
#define GET_TIME_OF_THE_DAY_ERROR -1
#define TIMEVAL_SUBSTRACT_ERROR -1

/* maximum number of bytes to capture */
#define SNAPLEN 3000
/* pcap_open_live read timeout in milliseconds */
#define TIMEOUT_PCAP_OPEN_LIVE 1000

/* w2lan packet types */
#define ETH_P_W2LAN_ANNOUNCE 0x6901
#define ETH_P_W2LAN_REQUEST 0x6902
#define ETH_P_W2LAN_DATA 0x6903

/* length of w2lan packets */
#define ETH_P_W2LAN_ANNOUNCE_LEN 34
#define ETH_P_W2LAN_REQUEST_LEN 20
/* see /usr/include/net/ethernet.h */
#define ETH_P_W2LAN_DATA_MIN_LEN ETHER_MIN_LEN
#define ETH_P_W2LAN_DATA_MAX_LEN ETH_FRAME_LEN
/* see /usr/include/linux/if_ether.h */
#define ETH_P_W2LAN_CONVERSATIONID_LEN ETH_ALEN


/* time between generating an announce and (possibly) sending the associated data */
/* during this time frame we accept requests */

// TODO adjust TIMEOUT_REQUEST
#define TIMEOUT_REQUEST_SECONDS 5
#define TIMEOUT_REQUEST_USECONDS 0


// TODO adjust W2LAN_ACTIVE_WINDOW
// Interval in (non w2lan)frames where w2lan reactive behaves like always (otherwise like passive)
#define MAX_NON_W2LAN_FRAMES 15

/* global variables */

/* Timer asociated with the period of receiving requests */
struct timeval *timerrequestglobal;
struct timeval *timerrequest;

/* w2lan operation mode (default = reactive) */
char *w2lanmode;

/* Flags to control the timer for reactive mode */
int counteractivewindow;
int activewindow;

/* Counter for testig purposes*/
int count;

/* global packet capture descriptor */
//static pcap_t *gpd;

/* our MAC address in host format */
u_int8_t myMAC[ETHER_ADDR_LEN];

/* w2lan protocol types (cal revisar format)*/
//unsigned char w2lan_announce_type[2] = "6901";
//unsigned char w2lan_request_type[2] = "6902";
//unsigned char w2lan_data_type[2] = "6903";

/* generic frame */
//char frame[MAX_ETHERNET_SIZE];

/* mirar mes endavant.... */
//static char * packet;

//const struct pcap_pkthdr* pkthdr;

/* global pointer to the conversationslist */
void* convl;
/* global pointer to the counterslist */
void* countl;
/* global pointer to the pendingframeslist */
void* pendingframesl;
/* global pointer to the timerslist */
void* timersl;

/* end global variables */

//long mac_addr_sys ( u_int8_t *addr, unsigned char * dvce );


#endif  /* _W2LAN_H_ */
