/***************************************************************************
 *   Copyright (C) 2007 by Francesc Burrull i Mestres.                     *
 *   francesc.burrull@upct.es                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _W2LAN_LISTS_H_
#include "w2lan_lists.h"
#endif

#include <assert.h>



/**************************************************************************/
/** conversations list ****************************************************/
/**************************************************************************/
/* conversationslist. It is a buffer of SIZEOF_CONVERSATIONSLIST elements */
/* each element is a conversation, consisting of:                         */
/*   conversationID: a six bytes conversation identifier                  */
/*   pending: a boolean indicating if the conversation is in process      */
/*   an ethernet header (mac dst, mac src, data type)                     */
/**************************************************************************/

conversationslist conversationslistcreate(void) {

   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: SIZE_CONVERSATIONSLIST = %d\n", SIZE_CONVERSATIONSLIST );
   }

   conversationslist tmp[SIZE_CONVERSATIONSLIST];
   int i;

   for ( i=0 ; i < SIZE_CONVERSATIONSLIST; i++ ) {
      tmp[i]=(conversationslist)malloc(sizeof(struct conversationslistElement));
      assert( tmp[i] != NULL );

      if (DEBUG) {
         printf("DEBUG: w2lan_lists.c: tmp[%d] = %d\n", i, (int)tmp[i] );
      }

   }

   for ( i=0 ; i < SIZE_CONVERSATIONSLIST-1; i++ ) {
      tmp[i]->next = tmp[i+1];
      if (DEBUG) {
         printf("DEBUG: w2lan_lists.c: tmp[%d]->next = %d\n", i, (int)(tmp[i]->next) );
      }
   }
   tmp[SIZE_CONVERSATIONSLIST-1]->next = tmp[0];
   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: tmp[%d]->next = %d\n", SIZE_CONVERSATIONSLIST-1 , (int)(tmp[SIZE_CONVERSATIONSLIST-1]->next) );
   }

   /*could be any between 0 and SIZE_CONVERSATIONSLIST-1 */
   return (conversationslist)tmp[SIZE_CONVERSATIONSLIST-1];
}

conversationslist conversationslistsearch( u_int8_t *conversationID , conversationslist list ) {

   int i;
   for ( i=0 ; i < SIZE_CONVERSATIONSLIST; i++ ) {
      if ( memcmp( list->conversationID, conversationID, ETH_P_W2LAN_CONVERSATIONID_LEN ) == 0 ) {
         return list;
      } else {
         list = list->next;
      }
   }
   return (conversationslist)NULL;
}


void conversationslistadd( u_int8_t *conversationID , u_int8_t * MACdst, u_int8_t * MACsrc ,  u_int16_t MACtype, conversationslist *list ) {

   memcpy( (*list)->conversationID , conversationID , ETH_P_W2LAN_CONVERSATIONID_LEN );
   (*list)->pending = 1;
   memcpy ( (*list)->header.MACdst , MACdst , ETHER_ADDR_LEN );
   memcpy ( (*list)->header.MACsrc , MACsrc , ETHER_ADDR_LEN );
   (*list)->header.MACtype = MACtype;

   (*list) = (*list)->next;
}


//void conversationslistget( u_int8_t *conversationID , u_int8_t * MACdst, u_int8_t * MACsrc ,  u_int16_t * MACtype, conversationslist list ) {
//
//   memcpy( conversationID , list->conversationID ,  ETH_P_W2LAN_CONVERSATIONID_LEN );
//   memcpy ( MACdst , list->header.MACdst , ETHER_ADDR_LEN );
//   memcpy ( MACsrc , list->header.MACsrc , ETHER_ADDR_LEN );
//   MACtype = list->header.MACtype;
//
//}



/* helper functions */

int conversationpending( conversationslist conv ) {
   return conv->pending;
}

void conversationsetpending( conversationslist conv ) {
   conv->pending = 1;
}

void conversationclearpending( conversationslist conv ) {
   conv->pending = 0;
}

u_int8_t* conversationgetmacdst( conversationslist conv ) {
   return conv->header.MACdst;
}

u_int8_t* conversationgetmacsrc( conversationslist conv ) {
   return conv->header.MACsrc;
}

u_int16_t conversationgetmactype( conversationslist conv ) {
   return conv->header.MACtype;
}

/**************************************************************************/
/**  counters list ********************************************************/
/**************************************************************************/

counterslist counterslistcreate(void) {
   return (counterslist)NULL;
}

counterslist counterslistsearch( u_int8_t *conversationID , counterslist list ) {

   if ( list == NULL ) {
      return NULL;
   } else {
      if ( memcmp( list->conversationID , conversationID, ETH_P_W2LAN_CONVERSATIONID_LEN ) == 0 ) {
          return list;
      } else {
          return counterslistsearch( conversationID, list->next );
      }
   }
}

void counterslistintoaslast( u_int8_t *conversationID , counterslist *list ) {

   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: counterslistintoaslast (*list) = %d\n", (int)(*list) );
   }
   /* the counter should not be in the list */
   assert( counterslistsearch(conversationID, *list) == NULL );

   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: counterslistintoaslast (*list) = %d\n", (int)(*list) );
   }
   if ( *list == NULL ) {
      *list = (counterslist)malloc( sizeof( struct counterslistElement ) );
      if (DEBUG) {
         printf("DEBUG: w2lan_lists.c: counterslistintoaslast (*list) = %d\n", (int)(*list) );
      }

      assert( *list != NULL );
      memcpy( (*list)->conversationID, conversationID, ETH_P_W2LAN_CONVERSATIONID_LEN );
      (*list)->requests = 0;
      (*list)->next = NULL;
   } else {
      counterslistintoaslast( conversationID, &((*list)->next) ); 
   }
}

int counterslistfirstinqueue( u_int8_t *conversationID , counterslist *list ) {

   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: counterslistfirstinqueue (*list) = %d\n", (int)(*list) );
   }
   /* counterslist should not be empty */
   assert( (*list) != NULL );
   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: counterslistfirstinqueue (*list) = %d\n", (int)(*list) );
   }

   int requests;
   counterslist tmp = (*list);
   memcpy( conversationID , (*list)->conversationID , ETH_P_W2LAN_CONVERSATIONID_LEN );
//   conversationID = (*list)->conversationID;
   requests = tmp->requests;
   (*list) = (*list)->next;
   free(tmp);
   return requests;
}

void counterslistincrease( u_int8_t *conversationID , counterslist list ) {

   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: counterslistincrease: (list) = %d\n", (int)list );
   }
   /* the counter should be in the list */
   assert( counterslistsearch(conversationID, list) != NULL );

   if ( memcmp( list->conversationID , conversationID, ETH_P_W2LAN_CONVERSATIONID_LEN ) == 0 ) {

      if (DEBUG) {
         printf("DEBUG: w2lan_lists.c: counterslistincrease before: list->requests = %d\n", (int)(list->requests) );
      }
      list->requests = list->requests + 1;
      if (DEBUG) {
         printf("DEBUG: w2lan_lists.c: counterslistincrease after: list->requests = %d\n", (int)(list->requests) );
      }

   } else { 
      counterslistincrease( conversationID , list->next );
   }
}



/**************************************************************************/
/**  pending frames list **************************************************/
/**************************************************************************/

pendingframeslist pendingframeslistcreate(void) {
   return (pendingframeslist)NULL;
}


pendingframeslist pendingframeslistsearch( u_int8_t *conversationID , pendingframeslist list ) {
   if ( list == NULL ) {
      return NULL;
   } else {
      if ( memcmp( list->conversationID , conversationID, ETH_P_W2LAN_CONVERSATIONID_LEN ) == 0 ) {
          return list;
      } else {
          return pendingframeslistsearch( conversationID, list->next );
      }
   }
}


void pendingframeslistput( u_int8_t *conversationID , u_int8_t *ethernetdatafield , u_int16_t size ,  pendingframeslist *list ) {

   /* the pending frame should not be in the list */
   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: pendingframeslistput (*list) = %d\n", (int)(*list) );
   }


   assert( pendingframeslistsearch(conversationID, *list) == NULL );
   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: pendingframeslistput (*list) = %d\n", (int)(*list) );
   }

   pendingframeslist tmp = *list;
   *list = (pendingframeslist)malloc( sizeof( struct pendingframeslistElement ) );

   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: pendingframeslistput (*list) = %d\n", (int)(*list) );
   }



   assert( *list != NULL );
   memcpy( (*list)->conversationID, conversationID, ETH_P_W2LAN_CONVERSATIONID_LEN );
   memcpy( (*list)->ethernetdatafield, ethernetdatafield, size );
   (*list)->size = size;
   (*list)->next = tmp;
}


void pendingframeslistextract( u_int8_t *conversationID , u_int8_t *ethernetdatafield , u_int16_t *size , pendingframeslist *list ) {

   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: pendingframeslistextract (*list) = %d\n", (int)(*list) );
   }
   /* the pending frame should be in the list */
   pendingframeslist pendingframe = pendingframeslistsearch(conversationID, *list);
   assert( pendingframe != NULL );

   if ( ( (*list) == pendingframe) ) {
      if (DEBUG) {
         printf("DEBUG: w2lan_lists.c: pendingframeslistextract before (*size) = %d\n", (*size) );
      }
      (*size) = (*list)->size;
      if (DEBUG) {
         printf("DEBUG: w2lan_lists.c: pendingframeslistextract after (*size) = %d\n", (*size) );
      }
      memcpy( ethernetdatafield , (*list)->ethernetdatafield , (*list)->size );
      pendingframeslist tmp = (*list)->next;
      free(*list);
      *list=tmp;
   } else {
      pendingframeslistextract( conversationID, ethernetdatafield, size, &((*list)->next) );
   }

}


/**************************************************************************/
/**  timers list **********************************************************/
/**************************************************************************/
// TODO
// Repasarla be. l'us del temps esta fet de qualsevol manera, per sortir del pas
//////////////////////////

timerslist timerslistcreate(void) {
   return (timerslist)NULL;
}

int timerslistempty( timerslist list ) {

   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: timerslistempty (list) = %d\n", (int)list );
   }
    if ( list == NULL) {
      return 1;
   } else {
      return 0;
   }
}

timerslist timerslistsearch( u_int8_t *conversationID , timerslist list ) {
   if ( list == NULL ) {
      return NULL;
   } else {
      if ( memcmp( list->conversationID , conversationID, ETH_P_W2LAN_CONVERSATIONID_LEN ) == 0 ) {
          return list;
      } else {
          return timerslistsearch( conversationID, list->next );
      }
   }
}

void timerslistintoaslast( u_int8_t *conversationID , struct timeval* t , timerslist *list ) {

   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: timerslistintoaslast (*list) = %d\n", (int)(*list) );
   }
   /* the timer should not be in the list */
   assert( timerslistsearch(conversationID, *list) == NULL );

   if ( *list == NULL ) {
      *list = (timerslist)malloc( sizeof( struct timerslistElement ) );
      assert( *list != NULL );
      memcpy( (*list)->conversationID, conversationID, ETH_P_W2LAN_CONVERSATIONID_LEN );
      (*list)->timer.tv_sec = t->tv_sec;
      (*list)->timer.tv_usec = t->tv_usec;
      (*list)->next = NULL;
   } else {
      timerslistintoaslast( conversationID, t, &((*list)->next) ); 
   }
}

void timerslistfirstinqueue( u_int8_t *conversationID , struct timeval* t , timerslist *list ) {

   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: timerslistfirstinqueue (*list) = %d\n", (int)(*list) );
   }
   /* timerslist should not be empty */
   assert( (*list) != NULL );

   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: timerslistfirstinqueue assert (*list) != NULL (%d)\n", (int)(*list) );
   }

   timerslist tmp = (*list);
   memcpy( conversationID , (*list)->conversationID , ETH_P_W2LAN_CONVERSATIONID_LEN );
//   conversationID = (*list)->conversationID;
   if (DEBUG) {
      printf( "DEBUG: w2lan_lists.c: timerslistfirstinqueue: conversationID: [ ");
      int i;
      for (i=0; i<ETHER_ADDR_LEN-1; ++i) {
        printf("%2.2x:", conversationID[i]);
      }
      printf("%2.2x ]\n", conversationID[ETHER_ADDR_LEN-1]);
   }
   t->tv_sec = tmp->timer.tv_sec;
   t->tv_usec = tmp->timer.tv_usec;
   (*list) = (*list)->next;
   if (DEBUG) {
      printf("DEBUG: w2lan_lists.c: timerslistfirstinqueue free(tmp)\n");
   }
   free(tmp);
   return;
}

void timerslistput( u_int8_t *conversationID , u_int8_t *ethernetdatafield , u_int16_t size ,  timerslist *list ) {

   /* the pending frame should not be in the list */
   assert( timerslistsearch(conversationID, *list) == NULL );

   timerslist tmp = *list;
   *list = (timerslist)malloc( sizeof( struct timerslistElement ) );
   assert( *list != NULL );
   memcpy( (*list)->conversationID, conversationID, ETH_P_W2LAN_CONVERSATIONID_LEN );
//   memcpy( (*list)->ethernetdatafield, ethernetdatafield, size );
   (*list)->size = size;
   (*list)->next = tmp;
   return;
}

void timerslistsuccessor( u_int8_t *conversationID , struct timeval *t, timerslist list ) {

   /* timerslist should not be empty */
   assert( list != NULL );

   t->tv_sec = list->timer.tv_sec;
   t->tv_usec = list->timer.tv_usec;
   return;
}

void timerslastinqueue( struct timeval *t, timerslist list ) {

   /* timerslist should not be empty */
   assert( list != NULL );

   while ( ( list->next ) != NULL ) {
      list = list->next;
   }
   t->tv_sec  = list->timer.tv_sec;
   t->tv_usec = list->timer.tv_usec;
   return;
}

// Auxiliar function
// Subtract the `struct timeval' values X and Y,
//   storing the result in RESULT.
//   Return 1 if the difference is negative, otherwise 0.

int timeval_subtract( struct timeval *result, struct timeval *x, struct timeval *y ) {

   /* Perform the carry for the later subtraction by updating y. */
   if (x->tv_usec < y->tv_usec) {
      int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
      y->tv_usec -= 1000000 * nsec;
      y->tv_sec += nsec;
   }
   if (x->tv_usec - y->tv_usec > 1000000) {
      int nsec = (y->tv_usec - x->tv_usec) / 1000000;
      y->tv_usec += 1000000 * nsec;
      y->tv_sec -= nsec;
   }
   /* Compute the time remaining to wait. tv_usec is certainly positive. */
   result->tv_sec = x->tv_sec - y->tv_sec;
   result->tv_usec = x->tv_usec - y->tv_usec;

   /* Return 1 if result is negative. */
   return x->tv_sec < y->tv_sec ;

}

void timeval_add( struct timeval *result, struct timeval *x, struct timeval *y ) {

   int usec = ( y->tv_usec + x->tv_usec );
   int sec  = (  y->tv_sec +  x->tv_sec );
   if ( usec >= 1000000 ) {
      usec = usec - 1000000;
      sec = sec + 1;
   }
   result->tv_usec = usec;
   result->tv_sec = sec;
   return;
}


